package jumpstart.max.business.commons.exception;

import java.io.Serializable;

import jumpstart.max.business.domain.base.BaseEntity;
import jumpstart.max.util.ClassUtil;
import jumpstart.max.util.MessageUtil;

@SuppressWarnings("serial")
public class ReferenceNotResettableException extends BusinessException {
	private String _entityLabelMessageId;
	private String _fieldLabelMessageId;
	private Serializable _referenceObjectId;
	private Serializable _newReferencedObjectId;

	public ReferenceNotResettableException(BaseEntity entity, String fieldLabelMessageId, Serializable referenceObjectId, 
		Serializable newReferencedObjectId) {

		// Don't convert the message ids to messages yet because we're in the server's locale, not the user's.

		super("ReferenceNotResettableException");
		_entityLabelMessageId = ClassUtil.extractUnqualifiedName(entity);
		_fieldLabelMessageId = fieldLabelMessageId;
		_referenceObjectId = referenceObjectId;
		_newReferencedObjectId = newReferencedObjectId;
	}
	
	@Override
	public String getMessage() {
	
		// We deferred converting the message ids to messages until now, when we are more likely to be in the user's locale.
	
		Object[] msgArgs = new Object[] { 
			MessageUtil.toText(_entityLabelMessageId), 
			MessageUtil.toText(_fieldLabelMessageId),
			_referenceObjectId, 
			_newReferencedObjectId 
		};
		
		String msg = MessageUtil.toText(_messageId, msgArgs);
		return msg;
	}

	public String getEntityLabelMessageId() {
		return _entityLabelMessageId;
	}

	public String getFieldLabelMessageId() {
		return _fieldLabelMessageId;
	}

	public Serializable getNewReferencedObjectId() {
		return _newReferencedObjectId;
	}

	public Serializable getReferenceObjectId() {
		return _referenceObjectId;
	}
}
