package jumpstart.max.business.commons.exception;

import jumpstart.max.business.domain.base.BaseEntity;
import jumpstart.max.util.ClassUtil;
import jumpstart.max.util.MessageUtil;

@SuppressWarnings("serial")
public class ReferencesWrongSubsetException extends BusinessException {
	private String _entityLabelMessageId;
	private String _fieldLabelMessageId;
	private String _referencedValueDescription;
	private String _referencedEntityLabelMessageId;
	private String _expectedSubsetDescription;
	private String _foundSubsetDescription;

	public ReferencesWrongSubsetException(BaseEntity entity, String fieldLabelMessageId,
		String referencedValueDescription, String referencedEntityLabelMessageId,
		String expectedSubsetDescription, String foundSubsetDescription) {

		// Don't convert the message ids to messages yet because we're in the server's locale, not the user's.

		super("ReferencesWrongSubsetException");
		_entityLabelMessageId = ClassUtil.extractUnqualifiedName(entity);
		_fieldLabelMessageId = fieldLabelMessageId;
		_referencedValueDescription = referencedValueDescription;
		_referencedEntityLabelMessageId = referencedEntityLabelMessageId;
		_expectedSubsetDescription = expectedSubsetDescription;
		_foundSubsetDescription = foundSubsetDescription;
	}
	
	@Override
	public String getMessage() {
	
		// We deferred converting the message ids to messages until now, when we are more likely to be in the user's locale.
	
		Object[] msgArgs = new Object[] { 
			MessageUtil.toText(_entityLabelMessageId), 
			MessageUtil.toText(_fieldLabelMessageId),
			_referencedValueDescription,
			MessageUtil.toText(_referencedEntityLabelMessageId),
			_expectedSubsetDescription,
			_foundSubsetDescription
		};
		
		String msg = MessageUtil.toText(_messageId, msgArgs);
		return msg;
	}

	public String getEntityLabelMessageId() {
		return _entityLabelMessageId;
	}

	public String getExpectedSubsetDescription() {
		return _expectedSubsetDescription;
	}

	public String getFieldLabelMessageId() {
		return _fieldLabelMessageId;
	}

	public String getFoundSubsetDescription() {
		return _foundSubsetDescription;
	}

	public String getReferencedEntityLabelMessageId() {
		return _referencedEntityLabelMessageId;
	}

	public String getReferencedValueDescription() {
		return _referencedValueDescription;
	}
}
