package jumpstart.max.business.domain.base;

import java.io.Serializable;

import javax.persistence.Column;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.MappedSuperclass;
import javax.persistence.Version;

@MappedSuperclass
public abstract class BaseEntityAutoGenId extends BaseEntity {

	@Id
	@GeneratedValue(strategy=GenerationType.AUTO)
	@Column(name = "id", nullable = false)
	protected Long id;

	@Version
	@Column(name = "version", nullable = false)
	protected Integer version;

	// The need for an equals() method is discussed at http://www.hibernate.org/109.html
	
	@Override
	public boolean equals(Object obj) {

		if (this.getClass() != obj.getClass()) {
			return false;
		}

		if (obj == this) {
			return true;
		}

		if (obj instanceof BaseEntityAutoGenId) {
			BaseEntityAutoGenId other = (BaseEntityAutoGenId) obj;
			if (other.getId() == null)
				return false; // we can do this because of the == check earlier
			else
				return other.getId().equals(this.getId());
		}
		else {
			return false;
		}
	}

	// The need for a hashCode() method is discussed at http://www.hibernate.org/109.html
	
	@Override
	public int hashCode() {
		if (getId() == null)
			return super.hashCode();
		else
			return getId().hashCode();
	}

	@Override
	public Serializable getIdForMessages() {
		return getId();
	}

	public Long getId() {
		return id;
	}

	public Integer getVersion() {
		return version;
	}
}
