package jumpstart.max.business.domain.security;

import java.util.Collections;
import java.util.Date;
import java.util.HashSet;
import java.util.Set;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.PostLoad;
import javax.persistence.PrePersist;
import javax.persistence.PreRemove;
import javax.persistence.PreUpdate;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;
import javax.persistence.UniqueConstraint;

import jumpstart.max.business.commons.exception.AddWhatException;
import jumpstart.max.business.commons.exception.BusinessException;
import jumpstart.max.business.commons.exception.GenericBusinessException;
import jumpstart.max.business.commons.exception.ReferencesWrongSubsetException;
import jumpstart.max.business.commons.exception.RemoveWhatException;
import jumpstart.max.business.commons.exception.ValueRequiredException;
import jumpstart.max.business.domain.base.BaseEntityAutoGenId;
import jumpstart.max.business.domain.reference.Code;
import jumpstart.max.business.domain.reference.CodeGroup;
import jumpstart.max.util.StringUtil;

import org.joda.time.DateMidnight;

/**
 * The User entity.
 * 
 * @version Revision: 0.1, Date: 10/06/2006
 */
@Entity
@Table(name = "user", uniqueConstraints = { @UniqueConstraint(columnNames = { "login_id" }) })
@SuppressWarnings("serial")
public class User extends BaseEntityAutoGenId {
//	static private Log LOG = LogFactory.getLog(User.class);
	static public final String ADMIN_LOGINID = "admin";

	@Column(name = "login_id", length = 15, nullable = false)
	protected String loginId;

	@Column(name = "password", length = 32)
	protected String password;
	
	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "salutation_code")
	protected Code salutation;

	@Column(name = "first_name", length = 20, nullable = false)
	protected String firstName;

	@Column(name = "last_name", length = 20, nullable = false)
	protected String lastName;

	@Column(name = "email_address", length = 80)
	protected String emailAddress;

	@Column(name = "expiration_date")
	@Temporal(TemporalType.DATE)
	protected Date expirationDate;
	
	@Column(name="style_id")
	protected int styleId = 1;

	@Column(name="date_input_pattern", length = 12, nullable = false)
	protected String dateInputPattern;
	
	@Column(name="date_view_pattern", length = 30, nullable = false)
	protected String dateViewPattern;
	
	@Column(name="date_list_pattern", length = 30, nullable = false)
	protected String dateListPattern;
	
	@Column(name="time4_view_pattern", length = 20, nullable = false)
	protected String time4ViewPattern;
	
	@Column(name="time4_list_pattern", length = 20, nullable = false)
	protected String time4ListPattern;
	
	@Column(name="time6_view_pattern", length = 20, nullable = false)
	protected String time6ViewPattern;
	
	@Column(name="time6_list_pattern", length = 20, nullable = false)
	protected String time6ListPattern;
	
	@Column(name="date_time4_view_pattern", length = 30, nullable = false)
	protected String dateTime4ViewPattern;
	
	@Column(name="date_time4_list_pattern", length = 30, nullable = false)
	protected String dateTime4ListPattern;
	
	@Column(name="date_time6_view_pattern", length = 30, nullable = false)
	protected String dateTime6ViewPattern;
	
	@Column(name="date_time6_list_pattern", length = 30, nullable = false)
	protected String dateTime6ListPattern;
	
	@Column(name="show_inspector")
	protected boolean showInspector = false;
	
	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "status_code", nullable = false)
	protected Code status;

	@OneToMany(mappedBy = "user", fetch = FetchType.LAZY, cascade = CascadeType.ALL)
	protected Set<UserRole> userRoles = new HashSet<UserRole>();

	@Transient
	protected transient String _loaded_password = null;

	public String toString() {
		StringBuffer buf = new StringBuffer();
		buf.append("User: [");
		buf.append("id=" + id + ", ");
		buf.append("loginId=" + loginId + ", ");
		buf.append("password=" + password + ", ");
		buf.append("salutation=" + toStringLazy(salutation, "getId") + ", ");
		buf.append("firstName=" + firstName + ", ");
		buf.append("lastName=" + lastName + ", ");
		buf.append("emailAddress=" + emailAddress + ", ");
		buf.append("expirationDate=" + expirationDate + ", ");
		buf.append("status=" + toStringLazy(status, "getId") + ", ");
		buf.append("styleId=" + styleId);
		buf.append("dateInputPattern=" + dateInputPattern);
		buf.append("dateViewPattern=" + dateViewPattern);
		buf.append("dateListPattern=" + dateListPattern);
		buf.append("time4ViewPattern=" + time4ViewPattern);
		buf.append("time4ListPattern=" + time4ListPattern);
		buf.append("time6ViewPattern=" + time6ViewPattern);
		buf.append("time6ListPattern=" + time6ListPattern);
		buf.append("dateTime4ViewPattern=" + dateTime4ViewPattern);
		buf.append("dateTime4ListPattern=" + dateTime4ListPattern);
		buf.append("dateTime6ViewPattern=" + dateTime6ViewPattern);
		buf.append("dateTime6ListPattern=" + dateTime6ListPattern);
		buf.append("showInspector=" + showInspector);
		buf.append("version=" + version);
		buf.append("]");
		return buf.toString();
	}

	@PrePersist
	void prePersistOfUser() throws BusinessException {
		validateUser();
	}

	@PostLoad
	void postLoadOfUser() {
		_loaded_password = password;
	}

	@PreUpdate
	void preUpdateOfUser() throws BusinessException {
		validateUser();
	}

	@PreRemove
	void preRemoveOfUser() throws BusinessException {
		// Check business rules here, eg.
		// if (entity.getParts().size() > 0) {
		// 	throw new CannotDeleteIsNotEmptyException(this, id, "Part");
		// }

		// There's no need to remove me from the parent(s) - that's the caller's
		// responsibility (and for performance, it might not bother)
	}

	public void validateUser() throws BusinessException {

		// Validate syntax...

		if (StringUtil.isEmpty(loginId)) {
			throw new ValueRequiredException(this, "User_loginId");
		}

		if (StringUtil.isEmpty(firstName)) {
			throw new ValueRequiredException(this, "User_firstName");
		}

		if (StringUtil.isEmpty(lastName)) {
			throw new ValueRequiredException(this, "User_lastName");
		}

		if (StringUtil.isEmpty(password)) {
			throw new ValueRequiredException(this, "User_password");
		}

		if (status == null) {
			throw new ValueRequiredException(this, "User_status");
		}

		// Validate semantics...

		if (expirationDate != null && loginId.equals(ADMIN_LOGINID)) {
			throw new GenericBusinessException("User_expirationdate_not_permitted_for_user", new Object[] { ADMIN_LOGINID });
		}

		if (!(status.getCodeGroup().getId().equals(CodeGroup.ID_USER_STATUS))) {
			throw new ReferencesWrongSubsetException(this, "User_status", "", "CodeGroup", CodeGroup.ID_USER_STATUS.toString(), status.getCodeGroup().getId().toString() );
		}

		if (!(status.getStatus().getId().equals(Code.ID_CODE_STATUS_ACTIVE))) {
			throw new ReferencesWrongSubsetException(this, "User_status", "", "Status", Code.ID_CODE_STATUS_ACTIVE.toString(), status.getStatus().getId().toString() );
		}

		if (salutation != null && !(salutation.getCodeGroup().getId().equals(CodeGroup.ID_USER_SALUTATION))) {
			throw new ReferencesWrongSubsetException(this, "User_salutation", "", "CodeGroup", CodeGroup.ID_USER_SALUTATION.toString(), status.getCodeGroup().getId().toString() );
		}

		if (salutation != null && !(salutation.getStatus().getId().equals(Code.ID_CODE_STATUS_ACTIVE))) {
			throw new ReferencesWrongSubsetException(this, "User_salutation", "", "Status", Code.ID_CODE_STATUS_ACTIVE.toString(), status.getStatus().getId().toString() );
		}

		if (dateInputPattern == null) {
			throw new ValueRequiredException(this, "User_dateInputPattern");
		}

		if (dateViewPattern == null) {
			throw new ValueRequiredException(this, "User_dateViewPattern");
		}

		if (dateListPattern == null) {
			throw new ValueRequiredException(this, "User_dateListPattern");
		}
		
		if (time4ViewPattern == null) {
			throw new ValueRequiredException(this, "User_time4ViewPattern");
		}

		if (time4ListPattern == null) {
			throw new ValueRequiredException(this, "User_time4ListPattern");
		}
		
		if (time6ViewPattern == null) {
			throw new ValueRequiredException(this, "User_time6ViewPattern");
		}

		if (time6ListPattern == null) {
			throw new ValueRequiredException(this, "User_stime6ListPattern");
		}
		
		if (dateTime4ViewPattern == null) {
			throw new ValueRequiredException(this, "User_sdateTime4ViewPattern");
		}

		if (dateTime4ListPattern == null) {
			throw new ValueRequiredException(this, "User_dateTime4ListPattern");
		}
		
		if (dateTime6ViewPattern == null) {
			throw new ValueRequiredException(this, "User_dateTime6ViewPattern");
		}

		if (dateTime6ListPattern == null) {
			throw new ValueRequiredException(this, "User_dateTime6ListPattern");
		}

	}

	public void authenticate(String password) throws BusinessException {

		if (password == null || !password.equals(this.password)) {
			throw new GenericBusinessException("User_password_incorrect");
		}

	}

	/**
	 * This method provides a way for users to change their own passwords.
	 */
	void changePassword(String currentPassword, String newPassword) throws BusinessException {

		if (this.password != null && !currentPassword.equals(this.password)) {
			throw new GenericBusinessException("User_password_incorrect");
		}
		
		setPassword(newPassword);
	}

	public String getLoginId() {
		return loginId;
	}

	public void setLoginId(String loginId) {
		this.loginId = loginId;
	}

	/**
	 * This method provides a way for security officers to "reset" the password.
	 */
	void setPassword(String newPassword) throws BusinessException {

		// TODO - review.  Do all tests of password size, content, history, etc here...

		if (StringUtil.isEmpty(newPassword)) {
			throw new ValueRequiredException(this, "User_password");
		}
		
		if (_loaded_password != null && newPassword.equals(_loaded_password)) {
			throw new GenericBusinessException("User_newpassword_is_same");
		}
		
		// The following is not at all secure. Remember that this object will be
		// detached and returned to the web client, where this clear-text
		// password might become vulnerable.
		// Alternatives include encrypting the password before storing it,
		// saving it encrypted to a separate entity that is never detached, or
		// using an enterprise-strength security framework like JAAS, Acegi,
		// etc.  MD5 encryption is planned for this example.

		this.password = newPassword;
	}
	
	/**
	 * addUserRole establishes my side of a bi-directional relationship owned by
	 * my child, UserRole. If detached, you MUST also call UserRole's
	 * setUser(user) . If attached, you might choose not to call this method for
	 * performance reasons, but be aware that as a consequence you would miss
	 * out on any business rule validaton in this method.
	 */
	public void addUserRole(UserRole userRole) throws BusinessException {

		// Check business rules here

		if (userRole == null) {
			throw new AddWhatException(this, id, "UserRole");
		}

		userRoles.add(userRole);
	}

	/**
	 * removeUserRole is typically used before removing the child from
	 * persistence. If attached, you might choose not to call this method for
	 * performance reasons, but be aware that as a consequence you would also
	 * miss out on any business rule validaton in this method.
	 */
	public void removeUserRole(UserRole userRole) throws BusinessException {

		// Check business rules here

		if (userRole == null) {
			throw new RemoveWhatException(this, id, "UserRole");
		}

		userRoles.remove(userRole);
	}

	public Set<UserRole> getUserRoles() {
		return Collections.unmodifiableSet(userRoles);
	}

	public String getFirstName() {
		return firstName;
	}

	public void setFirstName(String firstName) {
		this.firstName = firstName;
	}

	public String getLastName() {
		return lastName;
	}

	public void setLastName(String lastName) {
		this.lastName = lastName;
	}

	public String getEmailAddress() {
		return emailAddress;
	}

	public void setEmailAddress(String emailAddress) {
		this.emailAddress = emailAddress;
	}

	public Code getStatus() {
		return status;
	}

	public void setStatus(Code status) {
		this.status = status;
	}

	public Code getSalutation() {
		return salutation;
	}

	public void setSalutation(Code salutation) {
		this.salutation = salutation;
	}

	public DateMidnight getExpirationDate() {
		return (expirationDate == null ? null : new DateMidnight(expirationDate));
	}

	public void setExpirationDate(DateMidnight expirationDate) {
		this.expirationDate = (expirationDate == null ? null : expirationDate.toDate());
	}

	public String getDateInputPattern() {
		return dateInputPattern;
	}

	public void setDateInputPattern(String pattern) {
		this.dateInputPattern = pattern;
	}

	public String getDateListPattern() {
		return dateListPattern;
	}

	public void setDateListPattern(String pattern) {
		this.dateListPattern = pattern;
	}

	public String getDateTime4ListPattern() {
		return dateTime4ListPattern;
	}

	public void setDateTime4ListPattern(String pattern) {
		this.dateTime4ListPattern = pattern;
	}

	public String getDateTime6ListPattern() {
		return dateTime6ListPattern;
	}

	public void setDateTime6ListPattern(String pattern) {
		this.dateTime6ListPattern = pattern;
	}

	public String getTime4ListPattern() {
		return time4ListPattern;
	}

	public void setTime4ListPattern(String pattern) {
		this.time4ListPattern = pattern;
	}

	public String getTime6ListPattern() {
		return time6ListPattern;
	}

	public void setTime6ListPattern(String pattern) {
		this.time6ListPattern = pattern;
	}

	public String getDateViewPattern() {
		return dateViewPattern;
	}

	public void setDateViewPattern(String pattern) {
		this.dateViewPattern = pattern;
	}

	public String getDateTime4ViewPattern() {
		return dateTime4ViewPattern;
	}

	public void setDateTime4ViewPattern(String pattern) {
		this.dateTime4ViewPattern = pattern;
	}

	public String getDateTime6ViewPattern() {
		return dateTime6ViewPattern;
	}

	public void setDateTime6ViewPattern(String pattern) {
		this.dateTime6ViewPattern = pattern;
	}

	public String getTime4ViewPattern() {
		return time4ViewPattern;
	}

	public void setTime4ViewPattern(String pattern) {
		this.time4ViewPattern = pattern;
	}

	public String getTime6ViewPattern() {
		return time6ViewPattern;
	}

	public void setTime6ViewPattern(String pattern) {
		this.time6ViewPattern = pattern;
	}

	public boolean isShowInspector() {
		return showInspector;
	}

	public void setShowInspector(boolean showInspector) {
		this.showInspector = showInspector;
	}

	public int getStyleId() {
		return styleId;
	}

	public void setStyleId(int styleId) {
		this.styleId = styleId;
	}

}
