package jumpstart.max.web.pages.reference;
import java.util.List;

import jumpstart.max.business.commons.exception.ServiceException;
import jumpstart.max.business.commons.query.SearchOptions;
import jumpstart.max.business.domain.reference.Code;
import jumpstart.max.business.domain.reference.CodeGroup;
import jumpstart.max.business.domain.reference.iface.CodeSearchFields;
import jumpstart.max.business.domain.reference.iface.IReferenceFinderSvcLocal;
import jumpstart.max.business.domain.reference.iface.IReferenceManagerSvcLocal;
import jumpstart.max.web.commons.form.ObjectPropertySelectionModel;
import jumpstart.max.web.pages.base.ReturnableProtectedBasePage;

import org.apache.tapestry.annotations.Persist;
import org.apache.tapestry.engine.ILink;
import org.apache.tapestry.form.IPropertySelectionModel;

public abstract class CodeGroupCreatePage extends ReturnableProtectedBasePage {
	// static private final Log LOG = LogFactory.getLog(CodeGroupCreatePage.class);
	static public final String PAGE_NAME = "pages/reference/CodeGroupCreatePage";

	@Persist("client:form")
	public abstract CodeGroup getCodeGroup();
	public abstract void setCodeGroup(CodeGroup value);
	
	@Persist("session")
	// Cannot use client persistence because not serializable
	public abstract IPropertySelectionModel getStatusesModel();
	public abstract void setStatusesModel(IPropertySelectionModel value);

	public void activate() throws ServiceException {
		setCodeGroup(new CodeGroup());

		// Put a selection list of statuses onto the page
				
		IReferenceFinderSvcLocal referenceFinder1 = getBizServicesLocator().getReferenceFinderSvcLocal();
		CodeSearchFields search1 = new CodeSearchFields();
		search1.setCodeGroupId(CodeGroup.ID_CODEGROUP_STATUS);
		search1.setStatusId(Code.ID_CODE_STATUS_ACTIVE);
		List<Code> statuses = referenceFinder1.findCodesShallowish(search1, new SearchOptions());
		IPropertySelectionModel m1 = new ObjectPropertySelectionModel(statuses, Code.class, "getDescription", null, false, "");
		setStatusesModel(m1);

		getRequestCycle().activate(this);
	}

	public ILink doSave() {
		ILink redirectTo = null;

		if (!hasErrors()) {
			validateSave();

			if (!hasErrors()) {
				try {
					IReferenceManagerSvcLocal manager = getBizServicesLocator().getReferenceManagerSvcLocal();
    
					// Create the CodeGroup
    
					CodeGroup codeGroup = getCodeGroup();
					manager.createCodeGroup(codeGroup);
    
					redirectTo = popCallbackLink();
				}
				catch (Exception e) {
					handleBizServicesExceptionForCreate(e);
				}
			}
		}

		return redirectTo;
	}

	private void validateSave() {
		// if (xyz) {
		// recordError(getMessages().getMessage("Xyz_error"));
		// }
	}

	public void doCancel() {
		popAndActivateCallback();
	}

}
