package jumpstart.max.web.pages.security;

import java.util.List;

import jumpstart.max.business.commons.exception.ServiceException;
import jumpstart.max.business.domain.security.User;
import jumpstart.max.business.domain.security.UserRole;
import jumpstart.max.business.domain.security.iface.ISecurityFinderSvcLocal;
import jumpstart.max.web.pages.base.ReturnableProtectedBasePage;

import org.apache.tapestry.IExternalPage;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.annotations.Persist;

public abstract class MyProfileViewPage extends ReturnableProtectedBasePage implements IExternalPage {
	// static private final Log LOG = LogFactory.getLog(UserViewPage.class);
	static public final String PAGE_NAME = "pages/security/MyProfileViewPage";

	@Persist("session")
	// Probably best not to use client persistence, even though it encodes the
	// field, because User includes password
	public abstract User getUser();
	public abstract void setUser(User value);

	@Persist("client:form")
	public abstract List<UserRole> getUserRoles();
	public abstract void setUserRoles(List<UserRole> value);
	public abstract UserRole getUserRole();

	/*
	 * This method is used by callbacks - see
	 * ReturnableProtectedBasePage.popAndActivateCallback().
	 */
	public void activateExternalPage(Object[] parameters, IRequestCycle cycle) {
		this.activate();
	}

	public void activate() {
		try {

			// Get the User

			ISecurityFinderSvcLocal securityFinder = getBizServicesLocator().getSecurityFinderSvcLocal();
			Long id = getMyVisit().getMyUserId();
			User user = securityFinder.findUserShallowish(id);
			setUser(user);

			// Put a list of userRoles onto the page

			listUserRoles();

			getRequestCycle().activate(this);
			clearCallbacks();
		}
		catch (ServiceException e) {
			recordError(e.getLocalizedMessage());
		}
	}

	private void listUserRoles() {

		// Put a list of userRoles on the page

		ISecurityFinderSvcLocal securityFinder = getBizServicesLocator().getSecurityFinderSvcLocal();
		List<UserRole> l = securityFinder.findUserRolesShallowishByUser(getUser().getId());
		setUserRoles(l);
	}

	public void doEdit() {

		try {
			MyProfileEditPage nextPage = (MyProfileEditPage) getRequestCycle().getPage(MyProfileEditPage.PAGE_NAME);
			nextPage.activate();

			pushExternalCallback(this, new Object[] {});
		}
		catch (ServiceException e) {
			recordError(e.toString());
		}

	}

	public void doChangePassword() {

		try {
			MyPasswordChangePage nextPage = (MyPasswordChangePage) getRequestCycle().getPage(
					MyPasswordChangePage.PAGE_NAME);
			nextPage.activate();

			pushExternalCallback(this, new Object[] {});
		}
		catch (ServiceException e) {
			recordError(e.toString());
		}

	}

	public void doViewUserRole(Long id) {

		try {
			UserRoleViewPage nextPage = (UserRoleViewPage) getRequestCycle().getPage(UserRoleViewPage.PAGE_NAME);
			nextPage.activate(id);

			pushExternalCallback(this, new Object[] {});
		}
		catch (ServiceException e) {
			recordError(e.toString());
		}

	}

}
