package jumpstart.max.web.state;

import javax.naming.InitialContext;
import javax.naming.NamingException;

import jumpstart.max.business.domain.reference.iface.IReferenceFinderSvcLocal;
import jumpstart.max.business.domain.reference.iface.IReferenceManagerSvcLocal;
import jumpstart.max.business.domain.security.iface.ISecurityFinderSvcLocal;
import jumpstart.max.business.domain.security.iface.ISecurityManagerSvcLocal;

/**
 * ServiceLocator is used to centralize all JNDI lookups.
 * It is an Application-scoped State Object configured in hivemodule.xml.
 */

public class BizServicesLocator {
	// private static final Log LOG = LogFactory.getLog(Global.class);

	// Home interfaces are static, such that they are only
	// looked up once (JNDI lookup is very expensive).

	private IReferenceFinderSvcLocal _referenceFinderSvcLocal;
	private IReferenceManagerSvcLocal _referenceManagerSvcLocal;
	private ISecurityFinderSvcLocal _securityFinderSvcLocal;
	private ISecurityManagerSvcLocal _securityManagerSvcLocal;

	private InitialContext _initialContext;
	private boolean _servicesLocated = false;

	public BizServicesLocator() {
	}

	protected synchronized void locateServices() {
		if (!_servicesLocated) {
			try {
				_initialContext = new InitialContext();
				
				_referenceFinderSvcLocal = (IReferenceFinderSvcLocal) _initialContext.lookup("jumpstart-max/ReferenceFinderSvc/local");
				_referenceManagerSvcLocal = (IReferenceManagerSvcLocal) _initialContext.lookup("jumpstart-max/ReferenceManagerSvc/local");
				_securityFinderSvcLocal = (ISecurityFinderSvcLocal) _initialContext.lookup("jumpstart-max/SecurityFinderSvc/local");
				_securityManagerSvcLocal = (ISecurityManagerSvcLocal) _initialContext.lookup("jumpstart-max/SecurityManagerSvc/local");

				_servicesLocated = true;
			}
			catch (NamingException e) {
				_servicesLocated = false;
				clear();
				e.printStackTrace();
				throw new RuntimeException("JNDI lookup failed.  Is JBoss server not started?  Is jbossall-client.jar missing from classpath?", e);
			}
		}
	}

	public IReferenceFinderSvcLocal getReferenceFinderSvcLocal() {
		locateServices();
		return _referenceFinderSvcLocal;
	}

	public IReferenceManagerSvcLocal getReferenceManagerSvcLocal() {
		locateServices();
		return _referenceManagerSvcLocal;
	}

	public ISecurityFinderSvcLocal getSecurityFinderSvcLocal() {
		locateServices();
		return _securityFinderSvcLocal;
	}

	public ISecurityManagerSvcLocal getSecurityManagerSvcLocal() {
		locateServices();
		return _securityManagerSvcLocal;
	}

	/**
	 * Invoked after any kind of naming or remote exception. Cached naming contexts and interfaces
	 * are discarded.
	 */

	public synchronized void clear() {
		_initialContext = null;
		
		_referenceFinderSvcLocal = null;
		_referenceManagerSvcLocal = null;
		_securityFinderSvcLocal = null;
		_securityManagerSvcLocal = null;
	}

}
